// Run the following from Dev Console on Lichess.org

  const puzzles = ["LNSBK", "bxfBK", "P9pcW", "uJKIv", "BahQ4", "LZQqH", "nR9pr", "OJhB4", "UVsMd", "zpFdJ", "ONMBt", "Jtwhe", "RsPhh", "NDk5S", "R3bEt", "jyuWP", "uNHJd", "ynv2I", "SiDeS", "9ZeqA", "Lv6cM", "HxX7i", "1MCcu", "NwnxS", "Y1bBA", "Hf7sY", "EzXVN", "slQHf", "weo4a", "NPGSh", "Bfi5l", "GMtM5", "MRjiL"]

  const chunk = (n) => (xs) => xs.length < n ? [[...xs]] : [xs.slice(0, n), ...chunk(n)(xs.slice(n))]

  const getGame = (id) => 
    fetch(`https://lichess.org/api/game/${id}`).then(r => r.json())

  const format = ({p, g}) => {
    const date = new Date(new Date().getTime()).toISOString().replace(/\D/g, '')
    return {
      title: `Puzzle/${p.puzzle.id}`,
      tags: "Puzzle [[TTEC Player]]",
      created: date,
      modified: date,
      'black-handle': p.game.players[1].name,
      'black-rating': String(p.game.players[1].rating),
      'clock': p.game.clock,
      'game-date': new Date(g.createdAt).toISOString().replace(/\D/g, ''),
      'game-id': g.id,
      'game-url': `https://lichess.org/${g.id}/${p.puzzle.initialPly % 2 ? 'white' : 'black'}#${p.puzzle.initialPly + 1}`,
      perf: p.game.perf.name,
      pgn: p.game.pgn,
      'puzzle-id': p.puzzle.id,
      'puzzle-ply': String(p.puzzle.initialPly),
      'puzzle-rating': String(p.puzzle.rating),
      'puzzle-url': `https://lichess.org/training/${p.puzzle.id}`,
      status: g.status,
      themes: p.puzzle.themes.map(t => t.includes(' ') ? `[[${t}]]` : t).join(' '),
      'white-handle': p.game.players[0].name,
      'white-rating': String(p.game.players[0].rating),
      winner: g.winner,
    }
  }

  const getPuzzles = (puzzleIds) => console.log(`Fetching ["${puzzleIds.join('", "')}"]`) || Promise.all(puzzleIds.map(
    puz => fetch(`https://lichess.org/api/puzzle/${puz}`)
      .then(resp => resp.json())
      .then(p => getGame(p.game.id).then(g => ({p, g}))))
  ).then(rs => rs.map(format))

  const getAllPuzzles = (puzzleIds) => chunk(10)(puzzleIds).reduce(
    (pr, ids, i) =>
      console.log(`Scheduling ["${ids.join('", "')}"] in ${1e3 * i}ms`) ||
      pr.then(
        pzs => new Promise((res) => setTimeout(() => getPuzzles(ids).then(res), 1e3 * i)).then(npz => pzs.concat(npz))
    ), Promise.resolve([])
  )

  getAllPuzzles(puzzles)
    .then(r => JSON.stringify(r, null, 4))
    .then(console.log)
    .catch(err => console.warn(`Error: ${err}`))
